# -*- coding: us-ascii -*-
from zope.formlib.itemswidgets import MultiSelectWidget
from zope.formlib.widgets import TextAreaWidget


class TwoLineTextAreaWidget(TextAreaWidget):
    """ A two line text area widget. """

    height = 2


class MultiPreSelectCheckBoxWidget(MultiSelectWidget):
    """ A multi select check box widget that is pre selected. """

    def __init__(self, field, request):
        super(MultiPreSelectCheckBoxWidget, self).__init__(
            field, field.value_type.vocabulary, request)
        self.items = field.value_type.vocabulary.by_value.keys()

    def _getFormValue(self):
        return self.items

    def renderValue(self, value):
        rval = self.context.context.restrictedTraverse('@@searchreplacetable')
        return rval(self.context, self.request)



from collective.searchandreplace import SearchAndReplaceMessageFactory as _
from collective.searchandreplace.interfaces import ISearchReplaceSettings
from plone.app.registry.browser.controlpanel import ControlPanelFormWrapper
from plone.app.registry.browser.controlpanel import RegistryEditForm
from plone.z3cform import layout
from z3c.form import form


class SearchReplaceControlPanelForm(RegistryEditForm):
    form.extends(RegistryEditForm)
    schema = ISearchReplaceSettings


SearchReplaceControlPanelView = layout.wrap_form(
    SearchReplaceControlPanelForm, ControlPanelFormWrapper)
SearchReplaceControlPanelView.label = _(u"Search and Replace settings")



from collective.searchandreplace.interfaces import ISearchReplaceUtility
from collective.searchandreplace.testing import create_doc
from collective.searchandreplace.testing import edit_content
from collective.searchandreplace.testing import SEARCH_REPLACE_INTEGRATION_LAYER  # noqa
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from zope.component import getUtility

import unittest


class TestMatchCase(unittest.TestCase):
    """ Ensure that the match case flag works   """

    layer = SEARCH_REPLACE_INTEGRATION_LAYER

    def setUp(self):
        self.portal = self.layer['portal']
        self.srutil = getUtility(ISearchReplaceUtility)

    def testNoMatchCase(self):
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        create_doc(self.portal, id='doc1', title=u'Test Title',
                   text=u'Test Case')
        doc1 = getattr(self.portal, 'doc1')
        results = self.srutil.searchObjects(
            doc1,
            'test case',
            replaceText='foo',
            matchCase=False)
        self.assertEqual(len(results), 1)

    def testMatchCase(self):
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('Document', 'doc2')
        doc2 = getattr(self.portal, 'doc2')
        edit_content(doc2, title='test title', text='Test Case')
        results = self.srutil.searchObjects(
            doc2,
            'test case',
            replaceText='foo',
            matchCase=True)
        self.assertEqual(len(results), 0)

    def testOneMatch(self):
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('Document', 'doc1')
        doc1 = getattr(self.portal, 'doc1')
        edit_content(doc1, title='Test Title', text='Test Case')
        self.portal.invokeFactory('Document', 'doc2')
        doc2 = getattr(self.portal, 'doc2')
        edit_content(doc2, title='test title', text='test case')
        # path1 = '%s' %  '/'.join(doc1.getPhysicalPath())
        # path2 = '%s' % '/'.join(doc2.getPhysicalPath())
        # paths = {path1: {'body': [0]}, path2: {'body': [0]}}
        results = self.srutil.searchObjects(
            self.portal,
            'test case',
            replaceText='foo',
            matchCase=True,
            # searchItems=paths
        )
        self.assertEqual(len(results), 1)


class TestMultipleMatchCase(unittest.TestCase):
    """Ensure that multiple matches work."""

    layer = SEARCH_REPLACE_INTEGRATION_LAYER

    def setUp(self):
        self.portal = self.layer['portal']
        self.srutil = getUtility(ISearchReplaceUtility)
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('Document', 'doc1')
        self.doc1 = getattr(self.portal, 'doc1')
        edit_content(self.doc1,
                     title='Test test Test test',
                     text='Test test\nTest test')

    def testSearchCase(self):
        results = self.srutil.searchObjects(
            self.portal,
            'Test',
            replaceText='Bike',
            matchCase=True,
        )
        self.assertEqual(len(results), 4)
        # title field, first match
        self.assertEqual(results[0]['line'], 'title')
        self.assertEqual(results[0]['pos'], '0')
        # title field, second match
        self.assertEqual(results[1]['line'], 'title')
        self.assertEqual(results[1]['pos'], '10')
        # text field, line 1
        self.assertEqual(results[2]['line'], 'text 1')
        self.assertEqual(results[2]['pos'], '0')
        # text field, line 2
        self.assertEqual(results[3]['line'], 'text 2')
        self.assertEqual(results[3]['pos'], '10')

    def testSearchNoCase(self):
        results = self.srutil.searchObjects(
            self.portal,
            'Test',
            replaceText='Bike',
            matchCase=False,
        )
        self.assertEqual(len(results), 8)

    def testReplaceAll(self):
        from collective.searchandreplace.searchreplaceutility import getRawText
        results = self.srutil.searchObjects(
            self.portal,
            'Test',
            replaceText='Bike',
            matchCase=True,
            doReplace=True,
        )
        self.assertEqual(results, 4)
        self.assertEqual(self.doc1.Title(), 'Bike test Bike test')
        self.assertEqual(getRawText(self.doc1), 'Bike test\nBike test')

    def testReplacePaths(self):
        from collective.searchandreplace.searchreplaceutility import getRawText
        path = '/'.join(self.doc1.getPhysicalPath())
        paths = {
            path: {
                # Replace two positions from title
                'title': [0, 10],
                # Replace two positions from text
                'text': [5, 15],
            }
        }
        results = self.srutil.searchObjects(
            self.portal,
            'Test',
            replaceText='Bike',
            matchCase=False,
            doReplace=True,
            searchItems=paths,
        )
        self.assertEqual(results, 4)
        self.assertEqual(self.doc1.Title(), 'Bike test Bike test')
        self.assertEqual(getRawText(self.doc1), 'Test Bike\nTest Bike')



from collective.searchandreplace.interfaces import ISearchReplaceUtility
from collective.searchandreplace.testing import edit_content
from collective.searchandreplace.testing import SEARCH_REPLACE_INTEGRATION_LAYER  # noqa
from plone.app.testing import login
from plone.app.testing import logout
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from zope.component import getUtility

import unittest


class TestReplaceWhere(unittest.TestCase):
    """Ensure that we can replace in title, description and text."""

    layer = SEARCH_REPLACE_INTEGRATION_LAYER

    def setUp(self):
        self.portal = self.layer['portal']
        self.srutil = getUtility(ISearchReplaceUtility)

    def testReplaceText(self):
        from collective.searchandreplace.searchreplaceutility import getRawText
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('Document', 'doc1')
        doc1 = getattr(self.portal, 'doc1')
        edit_content(
            doc1,
            title='Test Title',
            description='Test Description',
            text='Test Case')
        self.assertEqual(getRawText(doc1), 'Test Case')
        # Search it.
        parameters = dict(
            context=doc1,
            find='test case',
            replaceText='foo',
            matchCase=False)
        results = self.srutil.searchObjects(**parameters)
        self.assertEqual(len(results), 1)
        self.assertEqual(getRawText(doc1), 'Test Case')
        # Replace it.
        parameters['doReplace'] = True
        results = self.srutil.searchObjects(**parameters)
        # Note: replacing returns an int, not a list.
        self.assertEqual(results, 1)
        self.assertEqual(getRawText(doc1), 'foo')
        # Other fields are not changed.
        self.assertEqual(doc1.Title(), 'Test Title')
        self.assertEqual(doc1.Description(), 'Test Description')

    def testReplaceTitle(self):
        from collective.searchandreplace.searchreplaceutility import getRawText
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('Document', 'doc2')
        doc1 = getattr(self.portal, 'doc2')
        edit_content(
            doc1,
            title='Test Title',
            description='Test Description',
            text='Test Case')
        self.assertEqual(doc1.Title(), 'Test Title')
        # Search it.
        parameters = dict(
            context=doc1,
            find='test title',
            replaceText='foo',
            matchCase=False)
        results = self.srutil.searchObjects(**parameters)
        self.assertEqual(len(results), 1)
        self.assertEqual(doc1.Title(), 'Test Title')
        # Replace it.
        parameters['doReplace'] = True
        results = self.srutil.searchObjects(**parameters)
        self.assertEqual(results, 1)
        self.assertEqual(doc1.Title(), 'foo')
        # Other fields are not changed.
        self.assertEqual(doc1.Description(), 'Test Description')
        self.assertEqual(getRawText(doc1), 'Test Case')

    def testReplaceDescription(self):
        from collective.searchandreplace.searchreplaceutility import getRawText
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('Document', 'doc1')
        doc1 = getattr(self.portal, 'doc1')
        edit_content(
            doc1,
            title='Test Title',
            description='Test Description',
            text='Test Case')
        self.assertEqual(doc1.Description(), 'Test Description')
        # Search it.
        parameters = dict(
            context=doc1,
            find='test desc',
            replaceText='foo',
            matchCase=False)
        results = self.srutil.searchObjects(**parameters)
        self.assertEqual(len(results), 1)
        self.assertEqual(doc1.Description(), 'Test Description')

        # Replace it.
        parameters['doReplace'] = True
        results = self.srutil.searchObjects(**parameters)
        self.assertEqual(results, 1)
        # Note: we have replaced only part of the description.
        self.assertEqual(doc1.Description(), 'fooription')

        # Other fields are not changed.
        self.assertEqual(doc1.Title(), 'Test Title')
        self.assertEqual(getRawText(doc1), 'Test Case')

    def testReplaceOnlyEditableContent(self):
        # setRoles(self.portal, TEST_USER_ID, ['Manager'])
        login(self.layer['app'], SITE_OWNER_NAME)
        # /mainfolder
        self.portal.invokeFactory('Folder', 'mainfolder')
        mainfolder = getattr(self.portal, 'mainfolder')
        edit_content(mainfolder, title='Test Title')

        # /mainfolder/maindoc
        mainfolder.invokeFactory('Document', 'maindoc')
        maindoc = getattr(mainfolder, 'maindoc')
        edit_content(maindoc, title='Test Title')

        # /mainfolder/subfolder
        mainfolder.invokeFactory('Folder', 'subfolder')
        subfolder = getattr(mainfolder, 'subfolder')
        edit_content(subfolder, title='Test Title')

        # /mainfolder/subfolder/subdoc
        subfolder.invokeFactory('Document', 'subdoc')
        subdoc = getattr(subfolder, 'subdoc')
        edit_content(subdoc, title='Test Title')

        # Give test user a local Editor role on the sub folder.
        subfolder.manage_addLocalRoles(TEST_USER_ID, ('Editor',))

        self.portal.portal_catalog.reindexObject(mainfolder)
        self.portal.portal_catalog.reindexObject(maindoc)
        self.portal.portal_catalog.reindexObject(subfolder)
        self.portal.portal_catalog.reindexObject(subdoc)

        # We are logged in as portal owner, so we can edit everything.
        results = self.srutil.searchObjects(
            mainfolder,
            'test title',
            replaceText='foo',
            matchCase=False)
        self.assertEqual(len(results), 4)

        # Login as the test user again.
        logout()
        login(self.portal, TEST_USER_NAME)
        # Now we can edit less: only the sub folder and sub doc.
        parameters = dict(
            context=mainfolder,
            find='test title',
            replaceText='foo',
            matchCase=False)
        results = self.srutil.searchObjects(**parameters)
        self.assertEqual(len(results), 2)
        paths = [x['path'] for x in results]
        self.assertIn('/'.join(subfolder.getPhysicalPath()), paths)
        self.assertIn('/'.join(subdoc.getPhysicalPath()), paths)
        self.assertNotIn('/'.join(mainfolder.getPhysicalPath()), paths)
        self.assertNotIn('/'.join(maindoc.getPhysicalPath()), paths)

        # Nothing has been changed, because we were only searching.
        self.assertEqual(mainfolder.Title(), 'Test Title')
        self.assertEqual(maindoc.Title(), 'Test Title')
        self.assertEqual(subfolder.Title(), 'Test Title')
        self.assertEqual(subfolder.Title(), 'Test Title')

        # Now replace instead of only searching.
        parameters['doReplace'] = True
        results = self.srutil.searchObjects(**parameters)
        self.assertEqual(results, 2)
        self.assertEqual(mainfolder.Title(), 'Test Title')
        self.assertEqual(maindoc.Title(), 'Test Title')
        self.assertEqual(subfolder.Title(), 'foo')
        self.assertEqual(subfolder.Title(), 'foo')

    def testReplaceAllTextFields(self):
        from collective.searchandreplace.searchreplaceutility import getRawText
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        # Note: we use our sample type here, which has extra fields.
        # This currently is Archetypes when testing on Plone 4,
        # and Dexterity when testing on Plone 5.
        self.portal.invokeFactory('SampleType', 'doc1')
        doc1 = getattr(self.portal, 'doc1')
        edit_content(
            doc1,
            title='Test Title',
            description='Test Description',
            rich='Test Rich',
            plain='Test Plain',
            line='Test Line',
            unsearchable='Test Unsearchable',
        )

        # Test the initial values.
        self.assertEqual(doc1.Title(), 'Test Title')
        self.assertEqual(doc1.Description(), 'Test Description')
        self.assertEqual(getRawText(doc1, 'rich'), 'Test Rich')
        self.assertEqual(getRawText(doc1, 'plain'), 'Test Plain')
        self.assertEqual(getRawText(doc1, 'line'), 'Test Line')
        self.assertEqual(getRawText(doc1, 'unsearchable'), 'Test Unsearchable')

        # Search it.
        parameters = dict(
            context=doc1,
            find='Test',
            replaceText='Foo',
            matchCase=False)
        results = self.srutil.searchObjects(**parameters)
        self.assertEqual(len(results), 5)

        # Nothing should have changed.
        self.assertEqual(doc1.Title(), 'Test Title')
        self.assertEqual(doc1.Description(), 'Test Description')
        self.assertEqual(getRawText(doc1, 'rich'), 'Test Rich')
        self.assertEqual(getRawText(doc1, 'plain'), 'Test Plain')
        self.assertEqual(getRawText(doc1, 'line'), 'Test Line')
        self.assertEqual(getRawText(doc1, 'unsearchable'), 'Test Unsearchable')

        # Replace it.
        parameters['doReplace'] = True
        results = self.srutil.searchObjects(**parameters)
        self.assertEqual(results, 5)

        # Most fields should have changed.
        self.assertEqual(doc1.Title(), 'Foo Title')
        self.assertEqual(doc1.Description(), 'Foo Description')
        self.assertEqual(getRawText(doc1, 'rich'), 'Foo Rich')
        self.assertEqual(getRawText(doc1, 'plain'), 'Foo Plain')
        self.assertEqual(getRawText(doc1, 'unsearchable'), 'Foo Unsearchable')

        # But not the textline field.
        self.assertEqual(getRawText(doc1, 'line'), 'Test Line')

    def testReplaceUnsearchableTextFields(self):
        from collective.searchandreplace.searchreplaceutility import getRawText
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        # Note: we use our sample type here, which has extra fields.
        # This currently is Archetypes when testing on Plone 4,
        # and Dexterity when testing on Plone 5.
        self.portal.invokeFactory('SampleType', 'doc1')
        doc1 = getattr(self.portal, 'doc1')
        edit_content(
            doc1,
            title='Test Title',
            unsearchable='Test Unsearchable',
        )

        # Test the initial values.
        self.assertEqual(doc1.Title(), 'Test Title')
        self.assertEqual(getRawText(doc1, 'unsearchable'), 'Test Unsearchable')

        # Search it with onlySearchableText true.
        parameters = dict(
            context=doc1,
            find='Unsearchable',
            replaceText='Foo',
            onlySearchableText=True,
            matchCase=False)
        results = self.srutil.searchObjects(**parameters)
        self.assertEqual(len(results), 0)

        # Replace it with onlySearchableText true.
        parameters['doReplace'] = True
        results = self.srutil.searchObjects(**parameters)
        self.assertEqual(results, 0)

        # Nothing should have changed.
        self.assertEqual(getRawText(doc1, 'unsearchable'), 'Test Unsearchable')

        # Now search everything, so onlySearchableText false.
        parameters['onlySearchableText'] = False
        parameters['doReplace'] = False
        results = self.srutil.searchObjects(**parameters)
        self.assertEqual(len(results), 1)

        # Nothing should have changed.
        self.assertEqual(getRawText(doc1, 'unsearchable'), 'Test Unsearchable')

        # Replace it.
        parameters['doReplace'] = True
        results = self.srutil.searchObjects(**parameters)
        self.assertEqual(results, 1)
        self.assertEqual(getRawText(doc1, 'unsearchable'), 'Test Foo')

    def testReplaceRawHTML(self):
        from collective.searchandreplace.searchreplaceutility import getRawText
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('Document', 'doc1')
        doc1 = getattr(self.portal, 'doc1')
        edit_content(
            doc1,
            title='Test Title',
            text='My <strong>Test</strong> Case')
        self.assertEqual(getRawText(doc1), 'My <strong>Test</strong> Case')

        # Search only in SearchableText.
        parameters = dict(
            context=doc1,
            find='<strong>Test</strong>',
            replaceText='<em>Test</em>',
            onlySearchableText=True,
            matchCase=False)
        results = self.srutil.searchObjects(**parameters)
        self.assertEqual(len(results), 0)

        # Search everywhere.
        parameters['onlySearchableText'] = False
        results = self.srutil.searchObjects(**parameters)
        self.assertEqual(len(results), 1)
        self.assertEqual(getRawText(doc1), 'My <strong>Test</strong> Case')

        # Replace it.
        parameters['doReplace'] = True
        results = self.srutil.searchObjects(**parameters)
        # Note: replacing returns an int, not a list.
        self.assertEqual(results, 1)
        self.assertEqual(getRawText(doc1), 'My <em>Test</em> Case')



from collective.searchandreplace.interfaces import ISearchReplaceUtility
from collective.searchandreplace.testing import SEARCH_REPLACE_INTEGRATION_LAYER  # noqa
from zope.component import getUtility

import unittest


class TestUtilityHelpers(unittest.TestCase):
    """Test some helper methods from the utility."""

    layer = SEARCH_REPLACE_INTEGRATION_LAYER

    def setUp(self):
        self.portal = self.layer['portal']
        self.srutil = getUtility(ISearchReplaceUtility)

    def test_parseItems(self):
        # parseItems parses lines of format 'line:pos:path'.  line is title or
        # description or fieldname plus space plus line number.
        items = [
            'title:0:path1',
            'text 2:10:path2',
            'description 1:4:path1',
            'description 1:42:path1',
            'new 1:7:path2',
            'new 4:63:path2',
        ]
        result = self.srutil.parseItems(items)
        self.assertEqual(len(result), 2)
        self.assertTrue('path1' in result)
        self.assertTrue('path2' in result)
        self.assertTrue('title' in result['path1'])
        self.assertTrue('description' in result['path1'])
        self.assertTrue('text' in result['path2'])
        self.assertTrue('new' in result['path2'])
        self.assertEqual(result['path1']['title'], [0])
        self.assertEqual(result['path1']['description'], [4, 42])
        self.assertEqual(result['path2']['text'], [10])
        self.assertEqual(result['path2']['new'], [7, 63])

    def test_parseItems_broken(self):
        # We ignore broken instructions.
        items = [
            'title:0:path1',
            'too many spaces 2:10:path2',
            'okay 1:4:path3',
            'more:than:2:colons',
            'not:enoughcolons',
            'bad_position:a:path6',
            'no_space:63:path7',
            'okay_again 20:3000:path8',
        ]
        result = self.srutil.parseItems(items)
        # paths with broken instructions may end up in the result, but the
        # instructions will be empty.
        self.assertTrue(result)
        self.assertEqual(result.get('path1'), {'title': [0]})
        self.assertFalse(result.get('path2'))
        self.assertEqual(result.get('path3'), {'okay': [4]})
        self.assertFalse(result.get('colons'))
        self.assertFalse(result.get('enoughcolons'))
        self.assertFalse(result.get('path6'))
        self.assertFalse(result.get('path7'))
        self.assertEqual(result.get('path8'), {'okay_again': [3000]})



# -*- coding: us-ascii -*-
from Acquisition import aq_parent
from collective.searchandreplace import SearchAndReplaceMessageFactory as _
from collective.searchandreplace.browser.customwidgets import TwoLineTextAreaWidget  # noqa
from collective.searchandreplace.interfaces import ISearchReplaceUtility
from five.formlib.formbase import AddForm
from plone.app.layout.navigation.defaultpage import isDefaultPage
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.statusmessages.interfaces import IStatusMessage
from zope.component import getUtility
from zope.formlib.form import action
from zope.formlib.form import FormFields
from zope.interface import Interface
from zope.schema import Bool
from zope.schema import Int
from zope.schema import Text


def validate_searchreplaceform(form, action, data):
    """ Validate the Search/Replace form. """


class ISearchReplaceForm(Interface):
    """ Interface for Search and Replace form """

    findWhat = Text(
        title=_(u'Find What'),
        description=_(u'Enter the text to find.'),
        required=True)

    replaceWith = Text(
        title=_(u'Replace With'),
        description=_(u'Enter the text to replace the original text with.'),
        required=False)

    maxResults = Int(
        title=_(u'Maximum Number of Results'),
        description=_(
            u'Maximum number of results to show. '
            'Warning: this has no effect on how many found texts are '
            'replaced when you use the Replace button directly without '
            'using the Preview.'),
        default=None,
        required=False)

    searchSubfolders = Bool(
        title=_(u'Search Subfolders'),
        description=_(
            u'If checked, this will recursively search '
            'through any selected folders and their '
            'children, replacing at each level.'),
        default=True,
        required=True)

    matchCase = Bool(
        title=_(u'Match Case'),
        description=_(u'Check the box for a case sensitive search.'),
        default=False,
        required=True)

    onlySearchableText = Bool(
        title=_(u'Fast search'),
        description=_(
            u'Use the catalog to search, just like the search form does. '
            'This only finds keywords, not html tags. '
            'You might have some text fields that are not found this way, '
            'so if not checked, you may find more content, '
            'but it will be slower. '
            'Regardless of this setting, when at least one match is found, '
            'text in all text fields may be replaced.'),
        required=False,
        default=True,
    )


class SearchReplaceForm(AddForm):
    """ """

    @property
    def form_fields(self):
        form_fields = FormFields(ISearchReplaceForm)
        container = aq_parent(self.context)
        if not self.context.isPrincipiaFolderish and not isDefaultPage(
                container, self.context):
            form_fields = form_fields.omit('searchSubfolders')
        form_fields['findWhat'].custom_widget = TwoLineTextAreaWidget
        form_fields['replaceWith'].custom_widget = TwoLineTextAreaWidget
        return form_fields

    label = _(u'Search and Replace')
    description = _(u'Search and replace text found in documents.')
    template = ViewPageTemplateFile('pageform.pt')

    @action(_(u'Preview'),
            validator=None,
            name=u'Preview')
    def action_preview(self, action, data):
        """ Preview files to be changed. """
        self.form_reset = False

    @action(_(u'Replace'),
            validator=validate_searchreplaceform,
            name=u'Replace')
    def action_replace(self, action, data):
        """ Replace text for all files. """
        self.form_reset = False
        srutil = getUtility(ISearchReplaceUtility)
        if 'form.affectedContent' in self.request:
            # Do only the selected items
            # nitems = len(self.request['form.affectedContent'])
            items = srutil.parseItems(self.request['form.affectedContent'])
            nitems = 0
            for page_url, page_result in items.items():
                for field, indexes in page_result.items():
                    nitems += len(indexes)
            replaced = srutil.searchObjects(
                self.context,
                data['findWhat'],
                searchSubFolders=data.get('searchSubfolders', False),
                matchCase=data['matchCase'],
                replaceText=data['replaceWith'],
                doReplace=True,
                searchItems=items,
                onlySearchableText=data['onlySearchableText'],
            )
            IStatusMessage(self.request).addStatusMessage(
                _(u'Search text replaced in ${replaced} of ${items} '
                  'instance(s).',
                  mapping={'replaced': replaced, 'items': nitems}),
                type='info')
        else:
            # Do everything you can find
            replaced = srutil.searchObjects(
                self.context,
                data['findWhat'],
                searchSubFolders=data.get('searchSubfolders', False),
                matchCase=data['matchCase'],
                replaceText=data['replaceWith'],
                onlySearchableText=data['onlySearchableText'],
                doReplace=True)
            IStatusMessage(self.request).addStatusMessage(
                _(u'Search text replaced in all ${items} instance(s).',
                  mapping={'items': replaced}),
                type='info')

    @action(_(u'Reset'),
            validator=None,
            name=u'Reset')
    def action_reset(self, action, data):
        """ Reset the form fields to their defaults. """



from collective.searchandreplace.testing import SEARCH_REPLACE_FUNCTIONAL_LAYER
from plone.testing import layered
from unittest import TestSuite

import doctest


oflags = (doctest.ELLIPSIS |
          doctest.NORMALIZE_WHITESPACE)


def test_suite():
    suite = TestSuite()

    basicsearchtest = layered(doctest.DocFileSuite(
        'tests/basicsearch.txt',
        package='collective.searchandreplace',
        optionflags=oflags),
        layer=SEARCH_REPLACE_FUNCTIONAL_LAYER)

    searchavailabletest = layered(doctest.DocFileSuite(
        'tests/searchavailable.txt',
        package='collective.searchandreplace',
        optionflags=oflags),
        layer=SEARCH_REPLACE_FUNCTIONAL_LAYER)

    suite.addTests([
        basicsearchtest,
        searchavailabletest,
    ])

    return suite



from pkg_resources import get_distribution
from plone import api
from plone.app.testing import applyProfile
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing.layers import FunctionalTesting
from plone.app.testing.layers import IntegrationTesting
from plone.testing import z2


PLONE_VERSION = get_distribution('Products.CMFPlone').version
MAJOR_PLONE_VERSION = int(PLONE_VERSION[0])


class SearchReplaceLayer(PloneSandboxLayer):

    def setUpZope(self, app, configurationContext):
        import collective.searchandreplace
        self.loadZCML(package=collective.searchandreplace)
        self.loadZCML('testing.zcml', package=collective.searchandreplace)
        if MAJOR_PLONE_VERSION >= 5:
            import plone.app.contenttypes
            self.loadZCML(package=plone.app.contenttypes)
            import collective.dexteritytextindexer
            self.loadZCML(package=collective.dexteritytextindexer)
        else:
            # Needed for our Archetypes SampleType.
            z2.installProduct(app, 'collective.searchandreplace')

    def setUpPloneSite(self, portal):
        if MAJOR_PLONE_VERSION >= 5:
            applyProfile(portal, 'plone.app.contenttypes:default')
            applyProfile(
                portal, 'collective.searchandreplace:testing-dexterity')
        else:
            applyProfile(
                portal, 'collective.searchandreplace:testing-archetypes')

        applyProfile(portal, 'collective.searchandreplace:default')
        setRoles(portal, TEST_USER_ID, ['Manager'])
        create_doc(portal, text=u'Get Plone now')
        setRoles(portal, TEST_USER_ID, ['Member'])


SEARCH_REPLACE_FIXTURE = SearchReplaceLayer()
SEARCH_REPLACE_INTEGRATION_LAYER = IntegrationTesting(
    bases=(SEARCH_REPLACE_FIXTURE, ),
    name='SearchReplaceLayer:Integration')
SEARCH_REPLACE_FUNCTIONAL_LAYER = FunctionalTesting(
    bases=(SEARCH_REPLACE_FIXTURE, ),
    name='SearchReplaceLayer:Functional')


def rich_text(text):
    if MAJOR_PLONE_VERSION < 5:
        # Use standard Archetypes field.
        return text
    # Use dexterity text field.
    from plone.app.textfield.value import RichTextValue
    return RichTextValue(
        raw=text,
        mimeType='text/html',
        outputMimeType='text/html',
    )


def create_doc(container, id='page', title=u'Title of page', text=u''):
    text = rich_text(text)
    api.content.create(container, 'Document', id=id, title=title,
                       text=text)


def edit_content(context, title=None, description=None, text=None,
                 rich=None, plain=None, line=None, unsearchable=None):
    if MAJOR_PLONE_VERSION >= 5:
        # Dexterity
        if title is not None:
            context.title = title
        if description is not None:
            context.description = description
        if text is not None:
            context.text = rich_text(text)
        if rich is not None:
            context.rich = rich_text(rich)
        if plain is not None:
            context.plain = plain
        if line is not None:
            context.line = line
        if unsearchable is not None:
            context.unsearchable = rich_text(unsearchable)
    else:
        # Archetypes
        if title is not None:
            context.setTitle(title)
        if description is not None:
            context.setDescription(description)
        if text is not None:
            context.setText(text)
        if rich is not None:
            context.setRich(rich)
        if plain is not None:
            context.setPlain(plain)
        if line is not None:
            context.setLine(line)
        if unsearchable is not None:
            context.setUnsearchable(unsearchable)
    context.reindexObject()



from collective.searchandreplace import SearchAndReplaceMessageFactory as _
from zope import schema
from zope.interface import Interface

import zope.deferredimport


zope.deferredimport.initialize()


zope.deferredimport.deprecated(
    ("The ISearchReplaceable marker interface is no longer used. "
     "All types are now searched by default. "
     "You can configure this in @@searchreplace-controlpanel if needed."),
    ISearchReplaceable='zope.interface:Interface',
)


class ISearchReplaceUtility(Interface):
    """ Search and replace utility interface """

    def getItems():
        """ Get items for preview. """


class ISearchReplaceSettings(Interface):
    """Control panel settings for search and replace.
    """

    restrict_searchable_types = schema.Bool(
        title=_(u'Restrict the enabled types.'),
        description=_(
            u'If checked, only the enabled types are searched, '
            'otherwise all types are searched.'),
        required=False,
        default=False,
    )

    enabled_types = schema.List(
        title=_(u'List of types that are searched.'),
        description=_(
            u"When 'Restrict the enable types' is checked, "
            "only the selected types are searched. "
            "Otherwise this list is ignored."),
        value_type=schema.Choice(
            vocabulary='plone.app.vocabularies.PortalTypes',
        ),
        required=False,
        default=[
            'Collection',
            'Document',
            'Event',
            'File',
            'Folder',
            'Image',
            'News Item',
        ],
    )

    maximum_text_characters = schema.Int(
        title=_(u'Maximum text characters'),
        description=_(
            u'The maximum number of characters to show '
            'before and after the found text.'),
        required=False,
        default=50,
    )



from collective.searchandreplace.interfaces import ISearchReplaceSettings
from plone.dexterity.interfaces import IDexterityFTI
from plone.registry.interfaces import IRegistry
from zope.component import getUtilitiesFor
from zope.component import getUtility

import logging


PROFILE_ID = 'profile-collective.searchandreplace:default'
logger = logging.getLogger('collective.searchandreplace')


def dummy_upgrade_step(context):
    return


def run_actions_step(context):
    context.runImportStepFromProfile(PROFILE_ID, 'actions')


def run_rolemap_step(context):
    context.runImportStepFromProfile(PROFILE_ID, 'rolemap')


def run_registry_step(context):
    context.runImportStepFromProfile(PROFILE_ID, 'plone.app.registry')


def add_control_panel_and_upgrade_settings(context):
    # We have added an interface ISearchReplaceSettings and want a controlpanel
    # for it.  First we must remove the old maximum_text_characters setting,
    # because it did not belong to an interface yet.  If we keep the old
    # settings, the controlpanel gets confused, complaining that the record
    # does not exist.
    registry = getUtility(IRegistry)
    record_name = 'collective.searchandreplace.maximum_text_characters'
    maximum_text_characters = registry.get(record_name)
    if maximum_text_characters is not None:
        del registry.records[record_name]
    # Run the registry step, registering our interface
    context.runImportStepFromProfile(PROFILE_ID, 'plone.app.registry')
    # Register our control panel.
    context.runImportStepFromProfile(PROFILE_ID, 'controlpanel')
    # Restore the previous setting.
    if maximum_text_characters is not None:
        # Use the new interface to set the value.
        settings = registry.forInterface(ISearchReplaceSettings)
        settings.maximum_text_characters = maximum_text_characters


def remove_behavior(context):
    # In version 6 we introduced a dexterity behavion, but in version 7 this is
    # no longer needed.  So we remove it from existing types, otherwise you get
    # warnings and errors in the logs when accessing a type.  It seems to cause
    # no harm.
    behavior_name = 'collective.searchandreplace.interfaces.ISearchReplaceable'
    for name, fti in getUtilitiesFor(IDexterityFTI):
        behaviors = list(fti.behaviors)
        if behavior_name not in behaviors:
            continue
        behaviors.remove(behavior_name)
        fti._updateProperty('behaviors', tuple(behaviors))
        logger.info('Removed behavior %s from type %s.', behavior_name, name)



from AccessControl import ClassSecurityInfo
from AccessControl.Permissions import add_documents_images_and_files
from App.class_init import InitializeClass
from Products.Archetypes.atapi import BaseContent
from Products.Archetypes.atapi import BaseSchema
from Products.Archetypes.atapi import LinesField
from Products.Archetypes.atapi import LinesWidget
from Products.Archetypes.atapi import listTypes
from Products.Archetypes.atapi import MetadataSchema
from Products.Archetypes.atapi import process_types
from Products.Archetypes.atapi import registerType
from Products.Archetypes.atapi import RichWidget
from Products.Archetypes.atapi import Schema
from Products.Archetypes.atapi import TextAreaWidget
from Products.Archetypes.atapi import TextField
from Products.CMFCore.utils import ContentInit
from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin


PROJECTNAME = 'collective.searchandreplace'

SampleTypeSchema = BaseSchema.copy() + MetadataSchema(()) + Schema((

    TextField(
        'rich',
        required=False,
        searchable=True,
        default_output_type='text/x-html-safe',
        widget=RichWidget(
            description='',
            label=u'Rich Text')
    ),

    TextField(
        'plain',
        required=False,
        searchable=True,
        widget=TextAreaWidget(
            description='',
            label=u'Plain Text')
    ),

    LinesField(
        'line',
        required=False,
        searchable=True,
        widget=LinesWidget(
            description='',
            label=u'Text Line')
    ),

    TextField(
        'unsearchable',
        required=False,
        searchable=False,
        default_output_type='text/x-html-safe',
        widget=RichWidget(
            description='',
            label=u'Unsearchable Text')
    ),

))


class SampleType(BaseContent, BrowserDefaultMixin):

    schema = SampleTypeSchema
    portal_type = 'SampleType'
    archetype_name = 'SampleType'
    security = ClassSecurityInfo()


InitializeClass(SampleType)
registerType(SampleType, PROJECTNAME)


def initialize_archetypes_testing(context):
    """Initializer called when initializing with testing.zcml."""
    listOfTypes = listTypes(PROJECTNAME)

    content_types, constructors, ftis = process_types(
        listOfTypes,
        PROJECTNAME)

    allTypes = zip(content_types, constructors)
    for atype, constructor in allTypes:
        kind = "%s: %s" % (PROJECTNAME, atype.archetype_name)
        ContentInit(
            kind,
            content_types=(atype,),
            permission=add_documents_images_and_files,
            extra_constructors=(constructor,),
        ).initialize(context)



##############################################################################
#    Copyright (c) 2009 Novell, All rights reserved.
#    Portions copyright 2009 Massachusetts Institute of Technology,
#    All rights reserved.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, version 2.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

from setuptools import setup, find_packages

version = '7.1'

setup(
    name='collective.searchandreplace',
    version=version,
    description="Batch Search and Replace",
    long_description=(open("README.rst").read() + "\n" +
                      open("CHANGES.rst").read()),
    # Get more strings from
    # https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Framework :: Plone",
        "Framework :: Plone :: 4.3",
        "Framework :: Plone :: 5.0",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    keywords='batch search replace',
    author='enPraxis',
    author_email='info@enpraxis.net',
    maintainer='Maurits van Rees',
    maintainer_email='m.van.rees@zestsoftware.nl',
    url='https://github.com/collective/collective.searchandreplace',
    license='GPL',
    packages=find_packages(exclude=['ez_setup']),
    namespace_packages=['collective'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'Acquisition',
        'five.formlib',
        'plone.app.layout',
        'plone.app.textfield',
        'plone.resource',
        'Products.CMFCore',
        'Products.CMFPlone',
        'Products.statusmessages',
        'setuptools',
        'Zope2',
        'zope.component',
        'zope.formlib',
        'zope.i18nmessageid',
        'zope.interface',
        'zope.publisher',
        'zope.schema',
    ],
    extras_require={
        'test': [
            'collective.dexteritytextindexer',
            'plone.app.testing',
            'plone.api',
        ],
    },
    entry_points="""
    # -*- Entry points: -*-
    [z3c.autoinclude.plugin]
    target = plone
    """,
)



# -*- coding: us-ascii -*-

from Acquisition import aq_base
from Acquisition import aq_parent
from collective.searchandreplace import SearchAndReplaceMessageFactory as _
from collective.searchandreplace.interfaces import ISearchReplaceSettings
from plone.app.layout.navigation.defaultpage import isDefaultPage
from plone.app.textfield import RichTextValue
from plone.app.textfield.interfaces import IRichText
from plone.dexterity.utils import iterSchemata
from plone.registry.interfaces import IRegistry
from Products.Archetypes.interfaces import ITextField
from Products.CMFCore.permissions import ModifyPortalContent
from Products.CMFCore.utils import getToolByName
from zope.component import getUtility
from zope.i18n import translate
from zope.schema import getFieldsInOrder
from zope.schema.interfaces import IText
from zope.schema.interfaces import ITextLine

import logging
import re


logger = logging.getLogger('collective.searchreplace')
searchflags = re.DOTALL | re.UNICODE | re.MULTILINE
# List of text fields that are handled separately, instead of together with all
# text fields.  Note that 'title' is a string field, so we handle it
# separately, being the only string field that we want to change.  But we list
# it here to be on the safe side.
CUSTOM_HANDLED_TEXT_FIELDS = [
    'title',
]


def _to_unicode(s):
    assert isinstance(s, basestring)
    if not isinstance(s, unicode):
        s = s.decode('utf-8')
    return s


class SearchReplaceUtility(object):
    """ Search and replace utility. """

    # Permission to check before modifying content.
    permission = ModifyPortalContent

    def searchObjects(self, context, find, **kwargs):
        """ Search objects and optionally do a replace. """
        # Get search parameters
        cpath = context.getPhysicalPath()
        if 'searchSubFolders' in kwargs:
            ssf = kwargs['searchSubFolders']
        else:
            ssf = True
        if 'matchCase' in kwargs:
            mc = kwargs['matchCase']
        else:
            mc = False
        onlySearchableText = kwargs.get('onlySearchableText', True)
        if 'replaceText' in kwargs:
            rtext = kwargs['replaceText']
            if rtext is None:
                # Allow replacing by an empty string.
                rtext = u''
        else:
            rtext = u''
        if 'doReplace' in kwargs:
            replace = kwargs['doReplace']
        else:
            replace = False
        if 'searchItems' in kwargs:
            sitems = kwargs['searchItems']
        else:
            sitems = None
        if 'maxResults' in kwargs:
            maxResults = kwargs['maxResults']
        else:
            maxResults = None
        # Get Regex matcher
        sflags = mc and searchflags or (searchflags | re.IGNORECASE)
        matcher = re.compile(find, sflags)
        # Get items to search
        query = {'query': '/'.join(cpath)}
        if context.isPrincipiaFolderish and not ssf:
            query['depth'] = 1
        container = aq_parent(context)
        if isDefaultPage(container, context) and ssf:
            query['query'] = '/'.join(container.getPhysicalPath())
        catalog = getToolByName(context, 'portal_catalog')
        parameters = dict(path=query)
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISearchReplaceSettings, check=False)
        if settings.restrict_searchable_types:
            parameters['portal_type'] = settings.enabled_types
        if onlySearchableText:
            parameters['SearchableText'] = '*{0}*'.format(find)
        brains = catalog(**parameters)
        memship = getToolByName(context, 'portal_membership')
        checkPermission = memship.checkPermission
        # Match objects
        results = []
        replaced = 0
        for b in brains:
            ipath = b.getPath()
            if not sitems or ipath in sitems:
                try:
                    obj = b.getObject()
                except (KeyError, AttributeError):
                    logger.warn('getObject failed for %s', ipath)
                    continue
                # Does the user have the modify permission on this object?
                if not checkPermission(self.permission, obj):
                    continue
                # If there is a filtered list of items, and it
                # is in the list, or if there is no filter
                # then process the item
                if replace:
                    # Do a replace
                    if sitems:
                        sitem = sitems[ipath]
                    else:
                        sitem = None
                    rep = self._replaceObject(matcher,
                                              obj,
                                              cpath,
                                              rtext,
                                              sitem)
                    if rep:
                        self._afterReplace(obj, find, rtext)
                        replaced += rep
                elif not replace:
                    # Just find the matches and return info
                    result = self._searchObject(matcher, obj)
                    if result:
                        results += result
                    if maxResults is not None and len(results) > maxResults:
                        results = results[:maxResults]
                        break
        if replace:
            return replaced
        else:
            return results

    def _afterReplace(self, obj, find, rtext):
        """Hook for doing things after a text has been replaced.

        - obj is the changed object
        - find is the found text
        - rtext is the replacement text

        By default, we will store a version in the CMFEditions repository.
        """
        repository = getToolByName(obj, 'portal_repository', None)
        if repository is None:
            return
        if obj.portal_type not in repository.getVersionableContentTypes():
            return
        comment = _(u'Replaced: ${old} -> ${new}',
                    mapping={'old': find, 'new': rtext})
        comment = translate(comment, context=obj.REQUEST)
        repository.save(obj, comment=comment)

    def _replaceObject(self, matcher, obj, cpath, rtext, mobjs):
        """ Replace text in objects """
        replaced = 0
        # rtext is already unicode
        base_obj = aq_base(obj)
        if mobjs:
            # Replace only the objects specified in mobjs
            title_positions = mobjs.pop('title', None)
            if title_positions is not None:
                title = _to_unicode(base_obj.Title())
                result = self._replaceText(matcher,
                                           title,
                                           rtext,
                                           title_positions)
                if result[0]:
                    replaced += result[0]
                    base_obj.setTitle(result[1])
            # Handle general text fields.
            for fieldname, positions in mobjs.items():
                text = self._getRawText(obj, fieldname)
                if text:
                    result = self._replaceText(matcher,
                                               text,
                                               rtext,
                                               positions)
                    if result[0]:
                        replaced += result[0]
                        self._setTextField(obj, fieldname, result[1])
        else:
            # Replace all occurences
            try:
                title = _to_unicode(base_obj.Title())
            except AttributeError:
                # Title might be acquired from parent for some types, which
                # breaks now that we have stripped away the acquisition chain
                # with aq_base.
                title = u''
            result = self._replaceText(matcher,
                                       title,
                                       rtext,
                                       None)
            if result[0]:
                replaced += result[0]
                base_obj.setTitle(result[1])
            text_fields = self._getTextFields(obj)
            for field in text_fields:
                text = self._getRawTextField(obj, field)
                if not text:
                    continue
                result = self._replaceText(matcher,
                                           text,
                                           rtext,
                                           None)
                if result[0]:
                    replaced += result[0]
                    self._setTextField(obj, field.__name__, result[1])

        # don't have to utf-8 encoding
        if replaced:
            obj.reindexObject()
        return replaced

    def _replaceText(self, matcher, text, rtext, indexes):
        """ Replace instances """
        newtext = ''
        mindex = 0
        replaced = 0
        mobj = matcher.finditer(text)
        for x in mobj:
            start, end = x.span()
            if not indexes or start in indexes:
                newtext += text[mindex:start]
                newtext += rtext
                mindex = end
                replaced += 1
        newtext += text[mindex:]
        return replaced, newtext

    def _searchObject(self, matcher, obj):
        """ Find location of search strings """
        results = []
        path = '/'.join(obj.getPhysicalPath())
        base = aq_base(obj)
        try:
            title = _to_unicode(base.Title())
        except AttributeError:
            # Title might be acquired from parent for some types, which breaks
            # now that we have stripped away the acquisition chain with
            # aq_base.
            title = u''
        mobj = matcher.finditer(title)
        for x in mobj:
            start, end = x.span()
            results.append({
                'path': path,
                'url': obj.absolute_url(),
                'line': 'title',
                'pos': '%d' % start,
                'text': self._getLinePreview(title,
                                             start,
                                             end), })
        text_fields = self._getTextFields(obj)
        if text_fields:
            for field in text_fields:
                text = self._getRawTextField(obj, field)
                if not text:
                    continue
                mobj = matcher.finditer(text)
                for x in mobj:
                    start, end = x.span()
                    results.append({
                        'path': path,
                        'url': obj.absolute_url(),
                        'line': '%s %d' % (field.__name__,
                                           self._getLineNumber(text, start)),
                        'pos': '%d' % start,
                        'text': self._getLinePreview(text,
                                                     start,
                                                     end), })
        return results

    def _getTextFields(self, obj):
        # Get all text fields, except ones that are handled separately.
        text_fields = []
        if getattr(aq_base(obj), 'Schema', None):
            # Archetypes
            for field in obj.Schema().values():
                if field.__name__ in CUSTOM_HANDLED_TEXT_FIELDS:
                    continue
                if not ITextField.providedBy(field):
                    continue
                text_fields.append(field)
        else:
            # Dexterity
            for schemata in iterSchemata(obj):
                fields = getFieldsInOrder(schemata)
                for name, field in fields:
                    if name in CUSTOM_HANDLED_TEXT_FIELDS:
                        continue
                    if IRichText.providedBy(field):
                        text_fields.append(field)
                        continue
                    # ITextLine inherits from IText.
                    # We want to replace in texts, but not textlines.
                    # Maybe this can be made configurable.
                    if ITextLine.providedBy(field):
                        continue
                    if not IText.providedBy(field):
                        continue
                    text_fields.append(field)
        return text_fields

    def _getField(self, obj, fieldname):
        return getField(obj, fieldname)

    def _getRawTextField(self, obj, field):
        return getRawTextField(obj, field)

    def _getRawText(self, obj, fieldname):
        return getRawText(obj, fieldname)

    def _setTextField(self, obj, fieldname, text):
        obj = aq_base(obj)
        if getattr(obj, 'Schema', None):
            # Archetypes
            field = obj.getField(fieldname)
            if field is None:
                logger.warn('Field %s not found for %s',
                            fieldname, obj.getId())
                return
            field.set(obj, text)
        else:
            # Dexterity
            field = self._getField(obj, fieldname)
            if field is None:
                logger.warn('Field %s not found for %s',
                            fieldname, obj.getId())
                return
            if IRichText.providedBy(field):
                # Get mimetype from old value.
                old = field.get(obj)
                if old is None:
                    text = RichTextValue(text)
                else:
                    text = RichTextValue(
                        text, old.mimeType, old.outputMimeType)
            field.set(obj, text)

    def _getLineNumber(self, text, index):
        return text.count('\n', 0, index) + 1

    def _getLinePreview(self, text, start, end):
        sindex = text[:start].rfind('\n')
        if -1 == sindex:
            sindex = 0
        eindex = text[end:].find('\n')
        if -1 == eindex:
            eindex = None
        else:
            eindex += end
        return (text[sindex:start],
                text[start:end],
                text[end:eindex])

    def parseItems(self, items):
        """ Get list of items from form values """
        itemd = {}
        if not isinstance([], type(items)):
            items = [items]
        for x in items:
            try:
                line, pos, path = x.split(':')
            except ValueError:
                continue
            if path not in itemd:
                itemd[path] = {}
            if 'title' == line:
                if 'title' not in itemd[path]:
                    itemd[path]['title'] = []
                itemd[path]['title'].append(int(pos))
            elif ' ' in line:
                try:
                    fieldname, line_number = line.split(' ')
                except ValueError:
                    continue
                if fieldname not in itemd[path]:
                    itemd[path][fieldname] = []
                itemd[path][fieldname].append(int(pos))
        return itemd


def getField(obj, fieldname):
    obj = aq_base(obj)
    if getattr(obj, 'Schema', None):
        # Archetypes
        return obj.getField(fieldname)
    # Dexterity
    for schemata in iterSchemata(obj):
        fields = getFieldsInOrder(schemata)
        for name, field in fields:
            if name == fieldname:
                return field


def getRawTextField(obj, field):
    text = None
    obj = aq_base(obj)
    if hasattr(field, 'getRaw'):
        # Archetypes
        baseunit = field.getRaw(obj, raw=True)
        if isinstance(baseunit, tuple):
            #  LinesField
            text = '\n'.join(baseunit)
        elif isinstance(baseunit.raw, unicode):
            text = baseunit.raw
        else:
            text = _to_unicode(field.getRaw(obj))
    else:
        # Dexterity
        baseunit = field.get(obj)
        if baseunit is None:
            text = u''
        else:
            # Rich text has a raw attribute, plain text simply has text
            # (unicode).
            text = getattr(baseunit, 'raw', baseunit)
    return text


def getRawText(obj, fieldname='text'):
    field = getField(obj, fieldname)
    if field is None:
        return u''
    return getRawTextField(obj, field)



from zope.i18nmessageid import MessageFactory


# Define permission string, so we can import it if needed.  See also
# the permission in configure.zcml.
UseSearchAndReplace = "collective.searchandreplace: Use Search And Replace"
# Message factory:
SearchAndReplaceMessageFactory = MessageFactory('SearchAndReplace')
